lists.openwall.net   lists  /  announce  owl-users  owl-dev  john-users  john-dev  passwdqc-users  yescrypt  popa3d-users  /  oss-security  kernel-hardening  musl  sabotage  tlsify  passwords  /  crypt-dev  xvendor  /  Bugtraq  Full-Disclosure  linux-kernel  linux-netdev  linux-ext4  linux-hardening  linux-cve-announce  PHC 
Open Source and information security mailing list archives
 
Hash Suite: Windows password security audit tool. GUI, reports in PDF.
[<prev] [next>] [<thread-prev] [thread-next>] [day] [month] [year] [list]
Message-ID: <20231023191532.2405326-3-seanjc@google.com>
Date:   Mon, 23 Oct 2023 12:15:29 -0700
From:   Sean Christopherson <seanjc@...gle.com>
To:     Sean Christopherson <seanjc@...gle.com>,
        Paolo Bonzini <pbonzini@...hat.com>
Cc:     kvm@...r.kernel.org, linux-kernel@...r.kernel.org,
        Michael Roth <michael.roth@....com>
Subject: [PATCH gmem 2/5] KVM: selftests: Handle memslot splits in private mem
 conversions test

Rework the SYNC_{SHARED,PRIVATE} handling in the private memory conversion
test to play nice with ranges that are covered by multiple memslots.  This
will allow letting the user specify the number of memslots, and more
importantly allow testing KVM_SET_MEMORY_ATTRIBUTES calls that also split
across memslots.

Opportunistically update the error messages to clarify whether the assert
fired in the guest vs. host.

Signed-off-by: Sean Christopherson <seanjc@...gle.com>
---
 .../kvm/x86_64/private_mem_conversions_test.c | 44 +++++++++++--------
 1 file changed, 25 insertions(+), 19 deletions(-)

diff --git a/tools/testing/selftests/kvm/x86_64/private_mem_conversions_test.c b/tools/testing/selftests/kvm/x86_64/private_mem_conversions_test.c
index c04e7d61a585..c3992a295b5a 100644
--- a/tools/testing/selftests/kvm/x86_64/private_mem_conversions_test.c
+++ b/tools/testing/selftests/kvm/x86_64/private_mem_conversions_test.c
@@ -28,25 +28,25 @@
 #define PER_CPU_DATA_SIZE	((uint64_t)(SZ_2M + PAGE_SIZE))
 
 /* Horrific macro so that the line info is captured accurately :-( */
-#define memcmp_g(gpa, pattern,  size)							\
-do {											\
-	uint8_t *mem = (uint8_t *)gpa;							\
-	size_t i;									\
-											\
-	for (i = 0; i < size; i++)							\
-		__GUEST_ASSERT(mem[i] == pattern,					\
-			       "Expected 0x%x at offset %lu (gpa 0x%llx), got 0x%x",	\
-			       pattern, i, gpa + i, mem[i]);				\
+#define memcmp_g(gpa, pattern,  size)								\
+do {												\
+	uint8_t *mem = (uint8_t *)gpa;								\
+	size_t i;										\
+												\
+	for (i = 0; i < size; i++)								\
+		__GUEST_ASSERT(mem[i] == pattern,						\
+			       "Guest expected 0x%x at offset %lu (gpa 0x%llx), got 0x%x",	\
+			       pattern, i, gpa + i, mem[i]);					\
 } while (0)
 
-static void memcmp_h(uint8_t *mem, uint8_t pattern, size_t size)
+static void memcmp_h(uint8_t *mem, uint64_t gpa, uint8_t pattern, size_t size)
 {
 	size_t i;
 
 	for (i = 0; i < size; i++)
 		TEST_ASSERT(mem[i] == pattern,
-			    "Expected 0x%x at offset %lu, got 0x%x",
-			    pattern, i, mem[i]);
+			    "Host expected 0x%x at gpa 0x%lx, got 0x%x",
+			    pattern, gpa + i, mem[i]);
 }
 
 /*
@@ -335,19 +335,25 @@ static void *__test_mem_conversions(void *__vcpu)
 		case UCALL_ABORT:
 			REPORT_GUEST_ASSERT(uc);
 		case UCALL_SYNC: {
-			uint8_t *hva = addr_gpa2hva(vm, uc.args[1]);
-			uint64_t size = uc.args[2];
+			uint64_t gpa  = uc.args[1];
+			size_t size = uc.args[2];
+			size_t i;
 
 			TEST_ASSERT(uc.args[0] == SYNC_SHARED ||
 				    uc.args[0] == SYNC_PRIVATE,
 				    "Unknown sync command '%ld'", uc.args[0]);
 
-			/* In all cases, the host should observe the shared data. */
-			memcmp_h(hva, uc.args[3], size);
+			for (i = 0; i < size; i += vm->page_size) {
+				size_t nr_bytes = min_t(size_t, vm->page_size, size - i);
+				uint8_t *hva = addr_gpa2hva(vm, gpa + i);
 
-			/* For shared, write the new pattern to guest memory. */
-			if (uc.args[0] == SYNC_SHARED)
-				memset(hva, uc.args[4], size);
+				/* In all cases, the host should observe the shared data. */
+				memcmp_h(hva, gpa + i, uc.args[3], nr_bytes);
+
+				/* For shared, write the new pattern to guest memory. */
+				if (uc.args[0] == SYNC_SHARED)
+					memset(hva, uc.args[4], nr_bytes);
+			}
 			break;
 		}
 		case UCALL_DONE:
-- 
2.42.0.758.gaed0368e0e-goog

Powered by blists - more mailing lists

Powered by Openwall GNU/*/Linux Powered by OpenVZ